package com.ozs.web.controller.pm;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.base.domain.BaseAgency;
import com.ozs.base.domain.BaseExpert;
import com.ozs.base.domain.vo.BaseExpertVo;
import com.ozs.base.service.BaseAgencyService;
import com.ozs.base.service.BaseExpertService;
import com.ozs.common.annotation.Log;
import com.ozs.common.constant.ModularConstans;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.enums.NameListType;
import com.ozs.common.enums.SysFileRefEnum;
import com.ozs.common.utils.StringUtils;
import com.ozs.pm.doman.PmDemand;
import com.ozs.pm.doman.PmReleaseAnnouncement;
import com.ozs.pm.doman.vo.requestVo.PmBaseExpertReqVo;
import com.ozs.pm.doman.vo.requestVo.PmDemandReqVo;
import com.ozs.pm.doman.vo.requestVo.PmPurchaseExecutionReqVo;
import com.ozs.pm.doman.vo.requestVo.PmReleaseAnnouncementReqVo;
import com.ozs.pm.doman.vo.responseVo.PmDemandResVo;
import com.ozs.pm.service.IPmDemandService;
import com.ozs.pm.service.PmReleaseAnnouncementService;
import com.ozs.system.domain.SysFileRef;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotEmpty;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * 采购执行Controller
 *
 * @author ruoyi
 * @date 2023-01-16
 */
@Api(tags = "采购执行")
@RestController
@RequestMapping("/pm/purchaseExecution")
public class PmPurchaseExecutionController extends BaseController {
    @Autowired
    private IPmDemandService pmDemandService;
    @Autowired
    private BaseAgencyService baseAgencyService;
    @Autowired
    private BaseExpertService baseExpertService;

    /**
     * 采购执行查询列表
     */
    @ApiOperation(value = "采购执行查询列表", notes = "参数非必传")
    @PostMapping("/list")
    public AjaxResult list(@RequestBody PmDemandReqVo pmDemandReqVo) {
        IPage<PmDemandResVo> page  = pmDemandService.selectPmDemandList(pmDemandReqVo,3);
        return success(page);
    }

    /**
     * 查询代理机构列表
     */
    @ApiOperation(value = "查询代理机构列表", notes = "采购需求ID必传")
    @PostMapping("/getAgencyList")
    public AjaxResult getAgencyList(@NotEmpty(message = "采购需求ID不能为空")
                               @RequestParam(value = "demandId", required = true) Long demandId) {
        PmDemand pmDemand = pmDemandService.getById(demandId);
        if(ObjectUtils.isEmpty(pmDemand)){
            return error("参数错误");
        }
        LambdaQueryWrapper<BaseAgency> lw = new LambdaQueryWrapper<>();
        lw.eq(BaseAgency::getCompanyType,pmDemand.getProjectType());
        lw.eq(BaseAgency::getStatus,0);
        List<BaseAgency> agencyList = baseAgencyService.list(lw);
        return success(agencyList);
    }

    /**
     * 选取代理-选择填写招标代理机构信息
     */
    @ApiOperation(value = "选取代理-选择填写招标代理机构信息",notes = "采购需求ID和代理机构ID必传")
    @PostMapping("/selectAgency")
    public AjaxResult selectAgency(@NotEmpty(message = "采购需求ID不能为空")
                                   @RequestParam(value = "demandId", required = true) Long demandId,
                                   @NotEmpty(message = "代理机构ID不能为空")
                                   @RequestParam(value = "agencyId", required = true) Long agencyId) {

        PmDemand pmDemand = pmDemandService.getById(demandId);
        if(ObjectUtils.isEmpty(pmDemand)){
            return error("参数错误");
        }
        BaseAgency baseAgency = baseAgencyService.getById(agencyId);
        if(ObjectUtils.isEmpty(baseAgency)){
            return error("参数错误");
        }
        PmDemand pmDemandUpdate = new PmDemand();
        pmDemandUpdate.setDemandId(demandId);
        pmDemandUpdate.setAgencyId(agencyId);
        pmDemandUpdate.setExtractAgencyTime(new Date());
        pmDemandUpdate.setUpdateTime(pmDemandUpdate.getExtractAgencyTime());
        pmDemandUpdate.setUpdateBy(getUserId().toString());
        return success(pmDemandService.updateById(pmDemandUpdate));
    }

    /**
     * 选取代理-抽取招标代理机构信息
     */
    @ApiOperation(value = "选取代理-抽取招标代理机构信息",notes = "采购需求ID必传")
    @PostMapping("/extractAgency")
    public AjaxResult extractAgency(@NotEmpty(message = "采购需求id不能为空")
                                     @RequestParam(value = "demandId", required = true) Long demandId) {

        PmDemand pmDemand = pmDemandService.getById(demandId);
        if(ObjectUtils.isEmpty(pmDemand)){
            return error("参数错误");
        }
        LambdaQueryWrapper<BaseAgency> lw = new LambdaQueryWrapper<>();
        lw.eq(BaseAgency::getCompanyType,pmDemand.getProjectType());
        lw.eq(BaseAgency::getStatus,0);
        Optional<BaseAgency> any = baseAgencyService.list(lw).stream().findAny();
        if(ObjectUtils.isEmpty(any)){
            return error("没有符合的招标代理机构，抽取失败！");
        }
        PmDemand pmDemandUpdate = new PmDemand();
        pmDemandUpdate.setDemandId(demandId);
        pmDemandUpdate.setAgencyId(any.get().getId());
        pmDemandUpdate.setExtractAgencyTime(new Date());
        pmDemandUpdate.setUpdateTime(pmDemandUpdate.getExtractAgencyTime());
        pmDemandUpdate.setUpdateBy(getUserId().toString());
        return success(pmDemandService.updateById(pmDemandUpdate));
    }

    @ApiOperation(value = "上传招标文件",notes = "采购需求ID和上传附件必传")
    @PostMapping("/upLoadBidFile")
    public AjaxResult upLoadBidFile(@NotEmpty(message = "数据为空") @RequestBody PmPurchaseExecutionReqVo pmPurchaseExecutionReqVo) {
        Long demandId = pmPurchaseExecutionReqVo.getDemandId();
        if(ObjectUtils.isEmpty(demandId)){
            return error("参数错误");
        }

        PmDemand pmDemand = pmDemandService.getById(demandId);
        if(ObjectUtils.isEmpty(pmDemand)){
            return error("参数错误");
        }

        List<SysFileRef> sysFileRefs = pmPurchaseExecutionReqVo.getSysFileRefs();
        if(ObjectUtils.isEmpty(sysFileRefs)){
            return error("参数错误");
        }
        if (pmDemandService.uploadFile(demandId, SysFileRefEnum.PM_BID_FILE.getType(),sysFileRefs,getUserId().toString())) {
            return AjaxResult.success();
        } else {
            return error("上传失败");
        }
    }

    @ApiOperation(value = "发布公告")
    @PostMapping("/insertPmReleaseAnnouncement")
    public AjaxResult insertPmReleaseAnnouncement(@NotEmpty(message = "数据为空")
                                       @RequestBody PmReleaseAnnouncementReqVo pmReleaseAnnouncementReqVo) {
        pmReleaseAnnouncementReqVo.setCreateBy(getUserId().toString());
        pmReleaseAnnouncementReqVo.setCreateTime(new Date());
        pmReleaseAnnouncementReqVo.setUpdateBy(pmReleaseAnnouncementReqVo.getCreateBy());
        pmReleaseAnnouncementReqVo.setUpdateTime(pmReleaseAnnouncementReqVo.getCreateTime());
        return toAjax(pmDemandService.insertPmReleaseAnnouncement(pmReleaseAnnouncementReqVo));
    }

    @ApiOperation(value = "获取专家身份证号列表")
    @PostMapping("/getExpertIdNumberList")
    public AjaxResult getExpertIdNumberList(@NotEmpty(message = "采购需求id不能为空")
                                                @RequestParam(value = "demandId", required = true) Long demandId) {
        PmDemand pmDemand = pmDemandService.getById(demandId);
        if(ObjectUtils.isEmpty(pmDemand)){
            return error("参数错误");
        }

        LambdaQueryWrapper<BaseExpert> lw = new LambdaQueryWrapper<BaseExpert>();
        lw.eq(BaseExpert::getVarietyPurchase,pmDemand.getProjectType())
                .eq(BaseExpert::getStatus,NameListType.WHITE.getCode());
        BaseExpert baseExpert = baseExpertService.getOne(lw);
        return success(baseExpert);
    }

    @ApiOperation(value = "根据身份证号查询专家", notes = "必传 查询条件：身份证号")
    @PostMapping("/findExpertWithIdNumber")
    public AjaxResult findExpertWithIdNumber(@NotEmpty(message = "采购需求id不能为空")
                                                 @RequestParam(value = "demandId", required = true) Long demandId,
                                             @NotEmpty(message = "身份证号不能为空")
                                       @RequestParam(value = "idNumber", required = true) String idNumber) {
        PmDemand pmDemand = pmDemandService.getById(demandId);
        if(ObjectUtils.isEmpty(pmDemand)){
            return error("参数错误");
        }
        LambdaQueryWrapper<BaseExpert> lw = new LambdaQueryWrapper<BaseExpert>();
        lw.eq(BaseExpert::getIdNumber,idNumber);
        BaseExpert baseExpert = baseExpertService.getOne(lw);
        if(baseExpert != null){
            if(baseExpert.getStatus().equals(NameListType.WHITE.getCode())){
                return error("该专家属于黑名单");
            }
            if(pmDemand.getProjectType().equals(baseExpert.getVarietyPurchase())){
                return error("该专家所属采购品种和项目类型不匹配");
            }
        }

        return success(baseExpert);
    }


    @ApiOperation(value = "填写专家信息批量提交", notes = "必传 采购需求ID和专家信息列表,注意：若该专家已经在库里存在，需要传专家对象BaseExpert的ID")
    @PostMapping("/insertExpertBatch")
    @PreAuthorize("@ss.hasPermi('pm:purchaseExecution:insertExpertBatch')")
    @Log(title = ModularConstans.purchaseExecution, businessType = BusinessType.INSERT)
    public AjaxResult insertExpertBatch(@RequestBody PmBaseExpertReqVo pmBaseExpertReqVo) {
        if (ObjectUtils.isEmpty(pmBaseExpertReqVo) || ObjectUtils.isEmpty(pmBaseExpertReqVo.getDemandId()) || ObjectUtils.isEmpty(pmBaseExpertReqVo.getBaseExpertList())) {
            return error("参数错误");
        }
        try {
            return toAjax(pmDemandService.insertExpertBatch(pmBaseExpertReqVo));
        } catch (Exception e) {
            return error(e.getMessage());
        }

    }
}
